<?php

/**
 * @file
 * Documentation on this module's hooks.
 */

/**
 * Define a code pool where to upload codes into.
 *
 * @return array
 *   Return an array keyed on the code pool name.
 *   The sub-array needs to include the following keys:
 *     - title: The permission string to check against. It should be the
 *       primary persission to check if the user has access to your particular
 *       tool.
 *     - codes table name: The name of the table where the uploaded codes
 *       will be stored in. This table will be created on the fly with the
 *       correct schema.
 *   The sub-array may contain this key
 *     - codes validate callback: The validate callback to call that determines
 *       the criteria that needs to be meet before the codes are uploaded into
 *       the pool. If this key is not provided the default validation will be
 *       called which checks for unique codes in the pool.
 */
function hook_codes_pool_type_info() {
  $info = [
    'default_codes_pool' => [
      'title' => 'Default Codes Pool',
      'codes table name' => CODES_POOL_DEFAULT_TABLE,
      'codes validate callback' => 'codes_upload_validate_unique_codes_in_pool',
    ],
  ];

  return $info;
}
