<?php

/**
 * Class CodeUploadEntity.
 *
 * Cannot use the following properties directly, must access field array:
 * $this->{PROPERTY}[LANGUAGE_NONE][$index]['value'].
 *
 * @property $field_codes_pool_program_code
 * @property $field_codes_pool_file_upload
 * @property $field_codes_pool_type
 * @property $field_new_codes_program_code
 * @property $field_new_codes_file_upload
 * @property $field_new_codes_effective_date
 * @property $field_new_codes_pool_type
 */
class CodeUploadEntity extends Entity {

  /**
   * @return mixed
   */
  public function getFieldCodesPoolProgramCode() {
    return $this->field_codes_pool_program_code[LANGUAGE_NONE][0]['value'] ?? '';
  }

  /**
   * @return mixed
   */
  public function getFieldCodesPoolFileUpload() {
    return $this->field_codes_pool_file_upload[LANGUAGE_NONE][0]['value'] ?? '';
  }

  /**
   * @return mixed
   */
  public function getFieldCodesPoolType() {
    return $this->field_codes_pool_type[LANGUAGE_NONE][0]['value'] ?? '';
  }

  public function getCodesPoolTypeTitle() {
    $type = $this->getFieldCodesPoolType();
    $info = codes_pool_type_definitions();

    return $info[$type]['title'] ?? '';
  }

  public function getCodesPoolTypeTable($type = NULL) {
    $info = codes_pool_type_definitions();

    if ($type === NULL) {
      $type = $this->getFieldCodesPoolType();
    }

    return $info[$type]['codes table name'] ?? '';
  }

  /**
   * @return mixed
   */
  public function getFieldNewCodesProgramCode() {
    return $this->field_new_codes_program_code[LANGUAGE_NONE][0]['value'] ?? '';
  }

  /**
   * @return mixed
   */
  public function getFieldNewCodesFileUploadFid() {
    return $this->field_new_codes_file_upload[LANGUAGE_NONE][0]['fid'] ?? '';
  }

  public function getFieldNewCodesFileUpload() {
    return $this->field_new_codes_file_upload[LANGUAGE_NONE][0] ?? [];
  }

  /**
   * @return mixed
   */
  public function getFieldNewCodesEffectiveDate() {
    return $this->field_new_codes_effective_date[LANGUAGE_NONE][0]['value'] ?? '';
  }

  /**
   * @return mixed
   */
  public function getFieldNewCodesType() {
    return $this->field_new_codes_pool_type[LANGUAGE_NONE][0]['value'] ?? '';
  }

  /**
   * Setter for the codes pool program code field.
   *
   * @param string $program_code
   *
   * @return bool
   */
  public function setFieldCodesPoolProgramCode(string $program_code) {
    // Check that program_code is a string.
    if (empty($program_code)) {
      return FALSE;
    }

    $this->field_codes_pool_program_code[LANGUAGE_NONE][0]['value'] = $program_code;
    return TRUE;
  }

  /**
   * Setter for the codes pool file upload field.
   *
   * @param array $info
   *
   * @return bool
   */
  public function setFieldCodesPoolFileUpload($info = []) {
    // Check if we have info array indexes that we need.
    if (!isset($info['fid'], $info['uid'], $info['filename'], $info['uri'])) {
      return FALSE;
    }

    $this->field_codes_pool_file_upload[LANGUAGE_NONE][0] = $info;
    return TRUE;
  }

  /**
   * Setter for the codes pool type field.
   *
   * @param $pool_type
   *
   * @return bool
   */
  public function setFieldCodesPoolType($pool_type) {
    // If this pool type is part of the defined pool types allow it.
    $pool_definitions = codes_pool_type_definitions();
    if (!isset($pool_definitions[$pool_type])) {
      return FALSE;
    }

    $this->field_codes_pool_type[LANGUAGE_NONE][0]['value'] = $pool_type;
    return TRUE;
  }

  /**
   * Form callback for the codes upload entity.
   *
   * @param $form
   * @param $form_state
   * @param $entity \CodeUploadEntity
   * @param string $op
   * @param string $entity_type
   *
   * @return mixed
   */
  public static function form(&$form, &$form_state, $entity, $op = 'edit', $entity_type = 'codes_upload_entity') {
    field_attach_form($entity_type, $entity, $form, $form_state);

    $form['field_codes_pool_file_upload']['#access'] = FALSE;
    $form['field_new_codes_effective_date']['#access'] = FALSE;
    $form['field_new_codes_program_code']['#access'] = FALSE;
    $form['field_new_codes_pool_type']['#access'] = FALSE;
    $form['field_codes_pool_program_code']['#access'] = FALSE;

    if (!property_exists($form['#entity'], 'is_new') || !$form['#entity']->is_new) {
      $form['field_codes_pool_program_code']['#access'] = FALSE;
      $form['field_codes_pool_type']['#access'] = FALSE;

      $used_codes = $entity->allocatedCodesCount();
      $total_uploaded_codes = $entity->totalCodesCount();

      $allotment_text = '<label>Current Codes Pool Info:</label>';
      $allotment_text .= '<ul class="helper">';
      $allotment_text .= '<li style="margin-bottom: 3px"><strong>Pool type:</strong> ' . $entity->getCodesPoolTypeTitle() . '</li>';
      $allotment_text .= '<li style="margin-bottom: 3px"><strong>Allotment:</strong> ' . $total_uploaded_codes . ' codes</li>';
      $allotment_text .= '<li style="margin-bottom: 3px"><strong>Dispensed:</strong> ' . $used_codes . ' (' . floor(($total_uploaded_codes - $used_codes) / $total_uploaded_codes * 100) . '% remaining)</li>';
      $allotment_text .= '</ul>';

      $form['codes_info_text'] = [
        '#markup' => $allotment_text,
        '#weight' => '0',
      ];

    }

    $form['actions'] = array(
      '#type' => 'actions',
    );

    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save Entity'),
    );

    return $form;
  }

  /**
   * Form Validation for the code upload entity.
   *
   * @param $form
   * @param $form_state
   *
   * @return bool|void
   */
  public static function formValidate(&$form, &$form_state) {

    // Don't call the validate callback if removing a file.
    if (strpos($form_state['triggering_element']['#name'], 'remove_button') !== FALSE &&
      in_array('file_managed_file_submit', $form_state['triggering_element']['#submit'])) {
      return TRUE;
    }

    // Don't call the validate if Pressing Cancel when adding a new entity.
    if (strpos($form_state['triggering_element']['#name'], 'add-cancel') !== FALSE) {
      return TRUE;
    }

    $valid = TRUE;

    $parents_path = implode('][', $form['#parents']);
    $entity_values = drupal_array_get_nested_value($form_state['values'], $form['#parents']);

    // Validation for new entity (they are all required by default).
    if (property_exists($form['#entity'], 'is_new') && $form['#entity']->is_new) {
      $pool_type = $entity_values['field_codes_pool_type'][LANGUAGE_NONE][0]['value'];
      $fid = $entity_values['field_new_codes_file_upload'][LANGUAGE_NONE][0]['fid'];

      if (count(form_get_errors()) > 0) {
        return FALSE;
      }

      // All required fields are entered. Call the codes validate callback.
      $file_validate = self::validateFileUploadByPoolType($fid, $pool_type);

      if ($file_validate !== TRUE) {
        form_set_error($parents_path . '][field_codes_pool_file_upload][' . LANGUAGE_NONE . '][0', $file_validate);
        $valid = FALSE;
      }

      return $valid;
    }

    // Validation for existing entity (new fields being entered and shown)
    $new_fid = $entity_values['field_new_codes_file_upload'][LANGUAGE_NONE][0]['fid'];

    // If any of the above 3 fields are entered, require all fields.
    if (!empty($new_fid)) {
      $pool_type = $entity_values['field_codes_pool_type'][LANGUAGE_NONE][0]['value'] ?? '';

      if (empty($pool_type)) {
        form_set_error($parents_path . '][field_codes_pool_type][' . LANGUAGE_NONE . '][0', t('Codes Pool Type field is required'));
      }

      if (count(form_get_errors()) > 0) {
        return FALSE;
      }

      // Validate file uploaded, and program code unique?
      // All required fields are entered. Call the codes validate callback.
      $file_validate = self::validateFileUploadByPoolType($new_fid, $pool_type);

      if ($file_validate !== TRUE) {
        form_set_error($parents_path . '][field_new_codes_file_upload][' . LANGUAGE_NONE . '][0', $file_validate);
        $valid = FALSE;
      }

    }

    return $valid;
  }

  /**
   * Submit handler for the codes upload entity.
   *
   * @param $form
   * @param $form_state
   *
   * @return bool
   */
  public static function formSubmit(&$form, &$form_state) {
    global $user;
    $info_definitions = codes_pool_type_definitions();

    // Get the proper path for the form_state values.
    $entity_values = drupal_array_get_nested_value($form_state['values'], $form['#parents']);

    // The inline entity form handles settting the form fields. Only call
    // this if not in the inline entity form.
    if (!isset($form['#ief_id'])) {
      $entity = entity_ui_form_submit_build_entity($form, $form_state);
    }
    else {
      /* @var $entity \CodeUploadEntity */
      $entity = $form['#entity'];
    }

    $time = time();

    // New entity, set the created field - created_by is already set by drupal.
    if ($entity->created == 0) {
      $entity->created = $time;
    }
    else {
      // we're updating the entity, set the modified and modified by.
      $entity->modified = $time;
      $entity->modified_by = $user->uid;
    }

    $form_state['redirect'] = 'admin/structure/code_upload_entity';

    // New entity, check effective date.
    if (property_exists($form['#entity'], 'is_new') && $form['#entity']->is_new) {
      $pool_type = $entity_values['field_codes_pool_type'][LANGUAGE_NONE][0]['value'];
      $pool_info = $info_definitions[$pool_type] ?? [];
      $fid = $entity_values['field_new_codes_file_upload'][LANGUAGE_NONE][0]['fid'];

      self::createCodePoolTable($pool_info['codes table name']);

      unset($entity->field_new_codes_file_upload[LANGUAGE_NONE][0]);
      $entity->save();

      $entity->uploadCodesFromFile($fid, $pool_info['codes table name']);

      return TRUE;
    }

    $entity->transitionToNewCodes();

    // Save entity without creating a new revision.
    $entity->revision = FALSE;
    $entity->save();
  }

  /**
   * Uploads Codes given a file and the table where to upload the codes to.
   *
   * @param $fid
   * @param $table
   */
  public function uploadCodesFromFile($fid, $table) {
    $temp_table = CODES_POOL_TEMP_CSV_UPLOAD_TABLE;

    if ($fid == 0) {
      return;
    }

    // Upload csv file contents into the temp table.
    codes_pool_upload_csv_into_temporary_table($fid);

    // Set entity and revision id on the temp table.
    db_query("UPDATE {$temp_table} SET codes_upload_entity_pid = {$this->internalIdentifier()}, codes_upload_entity_revision_id = {$this->revision_id}");

    // Copy the temp table values into the correct codes pool.
    db_query("INSERT INTO {$table} SELECT * FROM {$temp_table}");

    // Drop the temp table.
    db_query("DROP TEMPORARY TABLE {$temp_table}");

    // Clean out all unallocated codes for this entity.
    $this->removeUnallocatedCodes();
  }

  /**
   * Helper function to copy field values from the new fields to default fields.
   */
  public function transitionFieldValuesToDefault() {
    $this->setFieldCodesPoolProgramCode($this->getFieldNewCodesProgramCode());
    $this->setFieldCodesPoolFileUpload($this->getFieldNewCodesFileUpload());
    $this->setFieldCodesPoolType($this->getFieldNewCodesType());

    // Unset all the new fields.
    $this->field_new_codes_program_code = [];
    $this->field_new_codes_file_upload = [];
    $this->field_new_codes_effective_date = [];
    $this->field_new_codes_pool_type = [];
  }

  /**
   * The callback to transition codes that have reached the transition date.
   */
  public static function transitionToNewCodesCronCallback() {
    // Get entities that have the new effective date not in the future.
    $query = new EntityFieldQuery();
    $result = $query->entityCondition('entity_type', 'codes_upload_entity')
      ->fieldCondition('field_new_codes_effective_date', 'value', strtotime('midnight next day'), '<')
      ->execute();

    if (isset($result['codes_upload_entity'])) {
      $code_entity_pids = array_keys($result['codes_upload_entity']);
      $code_entities = entity_load('codes_upload_entity', $code_entity_pids);

      foreach ($code_entities as $code_entity) {
        $code_entity->transitionToNewCodes();
      }
    }
  }

  /**
   * Does the actual upload and transitioning to new codes.
   *
   * @return bool
   */
  private function transitionToNewCodes() {
    $new_code_pool_type = $this->getFieldNewCodesType();
    $new_fid = $this->getFieldNewCodesFileUploadFid();

    if (empty($new_fid)) {
      return FALSE;
    }

    $pool_table = $this->getCodesPoolTypeTable($new_code_pool_type);

    // Create pool table.
    self::createCodePoolTable($pool_table);

    // Create a new revision! Upload codes and transition to them.
    $this->is_new_revision = TRUE;
    $this->default_revision = TRUE;
    $this->save();

    // UploadCodesFromfile.
    $this->uploadCodesFromFile($new_fid, $pool_table);

    // Copy new fields into the default entity fields, clear out new fields.
    $this->transitionFieldValuesToDefault();

    // Save entity without creating a new revision.
    $this->revision = FALSE;
    $this->save();
  }

  /**
   * Creates the code pool table if it doesn't exist yet.
   *
   * @param $table_name
   *   The table name to create.
   */
  public static function createCodePoolTable($table_name = NULL) {
    if ($table_name === NULL) {
      return;
    }

    if (db_table_exists($table_name)) {
      return;
    }

    $schema = drupal_get_schema(CODES_POOL_DEFAULT_TABLE);
    db_create_table($table_name, $schema);
  }

  /**
   * Calls the codes validate callback when uploading codes into a pool.
   *
   * @param $fid
   *   The file id that is being uploaded.
   * @param $pool_type
   *   The pool type where the codes are being uploaded to.
   *
   * @return bool|string
   */
  public static function validateFileUploadByPoolType($fid, $pool_type) {
    $info_definitions = codes_pool_type_definitions();
    $pool_info = $info_definitions[$pool_type] ?? [];

    if (empty($fid)) {
      return t('No codes were uploaded. The file uploaded may be empty or improperly formatted.');
    }

    if (empty($pool_info)) {
      return 'something went wrong message here.';
    }

    $validate_callback = $pool_info['codes validate callback'] ?? '';
    if (function_exists($validate_callback)) {
      $file_validate = $validate_callback($fid, $pool_info['codes table name']);
    }
    // Fallback to the default codes pool validation callback.
    else {
      $file_validate = codes_upload_validate_unique_codes_in_pool($fid, $pool_info['codes table name']);
    }

    return $file_validate;
  }

  /**
   * Remove all unallocated codes from the prior revision from the Codes Pool.
   */
  private function removeUnallocatedCodes() {
    $pid = $this->internalIdentifier();

    // Get latest revision and clean out.
    $latest_revision_pool_type = $this->getPriorRevisionPoolType();
    if (!$latest_revision_pool_type) {
      return;
    }

    // Get the first pool type.
    $latest_revision_pool_type = reset($latest_revision_pool_type);
    $pool_table = $this->getCodesPoolTypeTable($latest_revision_pool_type);

    $info = $this->getPriorRevision();
    if ($info) {
      $db_delete = db_delete($pool_table)
        ->condition('codes_upload_entity_revision_id', $info['revision_id'])
        ->condition('codes_upload_entity_pid', $pid)
        ->condition('status', 0)
        ->execute();

      watchdog('Codes Pool', 'removeUnallocatedCodes() removed :count codes.', array(':count' => print_r($db_delete, TRUE)));
    }

  }

  /**
   * Gets the pool type of the latest revision that had codes uploaded.
   *
   * @return array
   *   The pool type that the prior codes were uploaded to or empty array.
   */
  public function getPriorRevisionPoolType() {
    $info = $this->getPriorRevision();

    if ($info) {

      // Get code_pool_type_field.
      $codes_type_query = db_select('field_revision_field_codes_pool_type', 'f')
        ->fields('f', ['field_codes_pool_type_value'])
        ->condition('entity_id', $this->internalIdentifier())
        ->condition('revision_id', $info['revision_id'])
        ->execute();

      return $codes_type_query->fetchCol();
    }

    return [];
  }

  /**
   * Get the prior revision for a given entity.
   *
   * @return mixed
   */
  public function getPriorRevision() {
    $latest_revision = db_select('codes_upload_entity_revision', 'c')
      ->fields('c', ['pid', 'revision_id'])
      ->condition('pid', $this->internalIdentifier())
      ->condition('revision_id', $this->revision_id, '<>')
      ->orderBy('revision_id', 'DESC')
      ->execute();

    return $latest_revision->fetchAssoc();
  }

  /**
   * Get Count of the allocated codes for an upload entity.
   *
   * @return int
   *     The count of allocated codes.
   */
  public function allocatedCodesCount() {
    $pool_type = $this->getFieldCodesPoolType();
    $pool_table = $this->getCodesPoolTypeTable($pool_type);

    $result = db_query("SELECT count(*) AS count FROM {$pool_table}
      WHERE status = 1 AND
      codes_upload_entity_pid = {$this->internalIdentifier()} AND
      codes_upload_entity_revision_id = {$this->revision_id}"
    )->fetchAssoc();

    if (!$result) {
      return 0;
    }

    return (int) $result['count'];
  }

  /**
   * Get the total codes count for a given upload entity.
   *
   * @return int
   *   The total count of all the codes for this entity.
   */
  public function totalCodesCount() {
    $pool_type = $this->getFieldCodesPoolType();
    $pool_table = $this->getCodesPoolTypeTable($pool_type);

    $result = db_query("SELECT count(*) AS count FROM {$pool_table}
      WHERE codes_upload_entity_pid = {$this->internalIdentifier()} AND
      codes_upload_entity_revision_id = {$this->revision_id}"
    )->fetchAssoc();

    if (!$result) {
      return 0;
    }

    return (int) $result['count'];
  }

  /**
   * Returns whether the pool has codes available or not.
   *
   * @return bool
   */
  public function hasCodesAvailable() {
    $table = $this->getCodesPoolTypeTable();

    $result = db_query("SELECT code FROM {$table}
      WHERE codes_upload_entity_pid = :pid AND
      codes_upload_entity_revision_id = :revision_id AND
      status = 0 LIMIT 1",
      array(
        ':pid' => $this->internalIdentifier(),
        ':revision_id' => $this->revision_id,
      )
    )->fetchAssoc();

    return !empty($result);
  }

  /**
   * Returns a Code and sets it to consumed, or FALSE if no code available.
   *
   * @return bool|string
   */
  public function consumeCode() {
    if (!$this->hasCodesAvailable()) {
      return FALSE;
    }

    $table = $this->getCodesPoolTypeTable();

    $result = db_query("SELECT code FROM {$table}
      WHERE codes_upload_entity_pid = :pid AND
      codes_upload_entity_revision_id = :revision_id AND
      status = 0 LIMIT 1",
      array(
        ':pid' => $this->internalIdentifier(),
        ':revision_id' => $this->revision_id,
      )
    )->fetchAssoc();

    if (isset($result['code'])) {
      $claim_code = db_update($table)
        ->fields(array('status' => 1))
        ->condition('status', 0)
        ->condition('code', $result['code'])
        ->condition('codes_upload_entity_pid', $this->internalIdentifier())
        ->condition('codes_upload_entity_revision_id', $this->revision_id)
        ->execute();

      // Only return the code if we can set as used.
      if ($claim_code > 0) {
        return $result['code'];
      }
    }

    return FALSE;
  }

}
