<?php


class CodesPoolEntityInlineEntityFormController extends EntityInlineEntityFormController {

  /**
   * Returns the entity form to be shown through the IEF widget.
   *
   * When adding data to $form_state it should be noted that there can be
   * several IEF widgets on one master form, each with several form rows,
   * leading to possible key collisions if the keys are not prefixed with
   * $entity_form['#parents'].
   *
   * @param $entity_form
   *   The entity form.
   * @param $form_state
   *   The form state of the parent form.
   */
  public function entityForm($entity_form, &$form_state) {
    $entity = $entity_form['#entity'];
    $entity_form = CodeUploadEntity::form($entity_form, $form_state, $entity);

    // Apply changes according to the instance field settings.
    if (property_exists($entity_form['#entity'], 'is_new') && $entity_form['#entity']->is_new) {
      $entity_form['field_codes_pool_type'][LANGUAGE_NONE]['#default_value'][0] = $this->settings['default_codes_pool_type'];
      $entity_form['field_codes_pool_type']['#access'] = !$this->settings['hide_codes_pool_type_field'];
      $entity_form['field_codes_pool_program_code'][LANGUAGE_NONE][0]['value']['#title'] = t('!title', ['!title' => $this->settings['program_code_label']]);
    }
    else {
      $entity_form['field_new_codes_pool_type'][LANGUAGE_NONE]['#default_value'][0] = $this->settings['default_codes_pool_type'];
      $entity_form['field_new_codes_pool_type']['#access'] = !$this->settings['hide_codes_pool_type_field'];
      $entity_form['field_new_codes_program_code'][LANGUAGE_NONE][0]['value']['#title'] = t('New') . ' ' . t('!title', ['!title' => $this->settings['program_code_label']]);

      $entity_form['codes_info_text']['#markup'] = str_replace('<strong>Program Code:</strong>', '<strong>' . $this->settings['program_code_label'] . ':</strong>', $entity_form['codes_info_text']['#markup']);
    }

    return $entity_form;
  }

  /**
   * Validates the entity form.
   *
   * @param $entity_form
   *   The entity form.
   * @param $form_state
   *   The form state of the parent form.
   */
  public function entityFormValidate($entity_form, &$form_state) {
    CodeUploadEntity::formValidate($entity_form, $form_state);
  }

  /**
   * Handles the submission of an entity form.
   *
   * Prepares the entity stored in $entity_form['#entity'] for saving by copying
   * the values from the form to matching properties and, if the entity is
   * fieldable, invoking Field API submit.
   *
   * @param $entity_form
   *   The entity form.
   * @param $form_state
   *   The form state of the parent form.
   */
  public function entityFormSubmit(&$entity_form, &$form_state) {
    parent::entityFormSubmit($entity_form, $form_state);

    CodeUploadEntity::formSubmit($entity_form, $form_state);
  }

  /**
   * Provide field instance settings for the inline entity widget
   *
   * Can set the default codes pool type and show or hide the codes pool type
   * form field.
   *
   * @param $field
   * @param $instance
   *
   * @return array
   */
  public function settingsForm($field, $instance) {
    $form = parent::settingsForm($field, $instance);

    $form['program_code_label'] = [
      '#type' => 'textfield',
      '#title' => 'Program code Label',
      '#default_value' => $this->settings['program_code_label'] ?? t('Program Code'),
    ];

    $form['default_codes_pool_type'] = [
      '#type' => 'radios',
      '#options' => codes_pool_type_options(),
      '#title' => 'Default Code Pool Type',
      '#default_value' => $this->settings['default_codes_pool_type'] ?? 'default_codes_pool',
    ];

    $form['hide_codes_pool_type_field'] = [
      '#type' => 'checkbox',
      '#title' => 'Hide Default Code Pool Type Form Element',
      '#description' => t('Hide or show the Codes pool Type field'),
      '#default_value' => $this->settings['hide_codes_pool_type_field'] ?? 0,
    ];

    return $form;
  }

}
