<?php

/**
 * Class CodesPoolUploadSelectionHandler
 *
 * Selection Handler for referencing Code Upload Entities.
 */
class CodesPoolUploadSelectionHandler extends EntityReference_SelectionHandler_Generic {

  /**
   * Implements EntityReferenceHandler::getInstance().
   */
  public static function getInstance($field, $instance = NULL, $entity_type = NULL, $entity = NULL) {
    return new CodesPoolUploadSelectionHandler($field, $instance, $entity_type, $entity);
  }

  /**
   * Build an EntityFieldQuery to get referencable entities.
   */
  protected function buildEntityFieldQuery($match = NULL, $match_operator = 'CONTAINS') {
    $query = parent::buildEntityFieldQuery();

    // Filter by program code field.
    $query->fieldCondition('field_codes_pool_program_code', 'value', $match, $match_operator);

    // Filter to the selected code pool types.
    $restrict_to_pool_types = $this->field['settings']['handler_settings']['codes_pool_types'] ?? [];
    if (!empty($restrict_to_pool_types)) {
      // Restrict to pool type.
      $query->fieldCondition('field_codes_pool_type', 'value', array_keys(array_filter($restrict_to_pool_types)), 'IN');
    }

    return $query;
  }

  /**
   * Implements EntityReferenceHandler::settingsForm().
   */
  public static function settingsForm($field, $instance) {
    $form = parent::settingsForm($field, $instance);

    $form['codes_pool_types'] = [
      '#type' => 'checkboxes',
      '#options' => codes_pool_type_options(),
      '#title' => 'Show entities with these Code Pool Types',
      '#description' => t('If no checkboxes are checked, all Entities will be shown'),
      '#default_value' => $field['settings']['handler_settings']['codes_pool_types'],
    ];

    return $form;
  }

}
